/*
 *
 * dbmmanage - handles DBM password files for use with Apache Web Server.
 * Copyright (C) 1997  Gianluca Faieta.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Gianluca Faieta
 * Via Agostinone, 34
 * 65016 Montesilvano - PE (Italy)
 * gfaieta@hotmail.com
 *
 */


#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ndbm.h>



/*
 * Hacked from login-utils/passwd.c
 */
#define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')


int main(int argc, char *argv[])
{
    DBM *f; 
    datum key, content; 
	char salt[2];
	char buf[32];
	time_t tm;


	/*
	 * Checks the number of arguments
	 */
	if (argc < 2) {
		printf ("Usage:\n\t%s <DBMfile> <command> <key> <value>\n", argv[0]);
		printf ("\tcommands: view, delete, adduser\n\n");
		exit (1);
	}


	/*
	 * Command: view
	 */
	if (strcmp (argv[2], "view") == 0) {
		if (!(f = dbm_open (argv[1], O_RDONLY, 0644))) {
			printf ("Error while opening file: %s\n", argv[1]);
			exit (1);
		}
		key = dbm_firstkey (f);
		while (key.dptr) {
			strncpy (buf, key.dptr, key.dsize);
			buf[key.dsize] = '\0';
			printf ("%s:", buf);

			content = dbm_fetch (f, key);
			strncpy (buf, content.dptr, content.dsize);
			buf[content.dsize] = '\0';
			printf ("%s\n", buf);

			key = dbm_nextkey (f, key);
		}
	}
	
	
	/*
	 * Command: adduser
	 */
	if (strcmp (argv[2], "adduser") == 0) {
		if (!(f = dbm_open(argv[1], O_RDWR | O_CREAT, 0644))) {
			printf ("Error while opening file: %s\n", argv[1]);
			exit (1);
		}

		/*
		 * Get a random salt from the timer.
		 * Hacked from login-utils/passwd.c
		 */
		time(&tm);
		salt[0] = bin_to_ascii(tm & 0x3f);
		salt[1] = bin_to_ascii((tm >> 5) & 0x3f);
		salt[2] = '\0';

		key.dptr = argv[3];
		key.dsize = strlen(key.dptr);
		content.dptr = crypt (argv[4], salt);
		content.dsize = strlen(content.dptr);

		dbm_store (f, key, content, DBM_REPLACE);
		printf ("User %s added with password %s, encrypted to %s\n",
		        argv[3], argv[4], content.dptr);
	}


	/*
	 * Command: delete
	 */
	if (strcmp (argv[2], "delete") == 0) {
		if (!(f = dbm_open(argv[1], O_RDWR, 0644))) {
			printf ("Error while opening file: %s\n", argv[1]);
			exit (1);
		}

		key.dptr = argv[3];
		key.dsize = strlen(key.dptr);

		dbm_delete (f, key);
	}


    dbm_close(f);
    exit(0); 
}
